/**
 * Redistribution and use of this software and associated documentation
 * ("Software"), with or without modification, are permitted provided
 * that the following conditions are met:
 *
 * 1. Redistributions of source code must retain copyright
 *    statements and notices.  Redistributions must also contain a
 *    copy of this document.
 *
 * 2. Redistributions in binary form must reproduce the
 *    above copyright notice, this list of conditions and the
 *    following disclaimer in the documentation and/or other
 *    materials provided with the distribution.
 *
 * 3. The name "Exolab" must not be used to endorse or promote
 *    products derived from this Software without prior written
 *    permission of Exoffice Technologies.  For written permission,
 *    please contact info@exolab.org.
 *
 * 4. Products derived from this Software may not be called "Exolab"
 *    nor may "Exolab" appear in their names without prior written
 *    permission of Exoffice Technologies. Exolab is a registered
 *    trademark of Exoffice Technologies.
 *
 * 5. Due credit should be given to the Exolab Project
 *    (http://www.exolab.org/).
 *
 * THIS SOFTWARE IS PROVIDED BY EXOFFICE TECHNOLOGIES AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * EXOFFICE TECHNOLOGIES OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Copyright 1999 (C) Exoffice Technologies Inc. All Rights Reserved.
 *
 * $Id: ActionPluginEdit.java,v 1.2 2002/02/25 20:30:55 dblevins Exp $
 */
package org.openejb.ui.jedi.openejb11.ejb;

import java.awt.Component;
import org.opentools.deployer.common.StoreManager;
import org.opentools.deployer.plugins.EditAction;
import org.opentools.deployer.plugins.Editor;
import org.opentools.deployer.plugins.Entry;
import org.opentools.deployer.plugins.Plugin;
import org.openejb.ui.jedi.openejb11.ejb.gui.PluginEditor;

/**
 * Handles transferring data from metadata to GUI and back again for editing
 * top-level EJB JAR configuration information on screen.  Currently, there is
 * no such information, so the editor component is empty and this class is not
 * used.
 *
 * @author Aaron Mulder (ammulder@alumni.princeton.edu)
 * @version $Revision: 1.2 $
 */
public class ActionPluginEdit extends OpenEjbAction implements EditAction {

    public ActionPluginEdit(OpenEjbPlugin plugin) {
        super(plugin);
    }

    public String getName() {
        return "Edit Overall Configuration";
    }

    public String getDescription() {
        return "Edit the top-level configuration for this EJB JAR. "+
               "These settings apply to the JAR as a whole, and/or all "+
               "the objects within.";
    }

    public Editor execute(final Entry source) {
        final EntryPlugin ejb = (EntryPlugin)source;

        return new OpenEJBEditor() {
            PluginEditor editor = new PluginEditor();

            public Component getComponent() {
                OpenEjbMetaData data = (OpenEjbMetaData)ejb.getMetaData();

                editor.clearModified();
                return editor;
            }

            public boolean isModified() {
                boolean modified = editor.isModified();
                return modified;
            };

            public void save() {
                OpenEjbMetaData data = (OpenEjbMetaData)ejb.getMetaData();
                editor.clearModified();
            }

            public void close() {
                editor = null;
            }

            public EditAction getAction() {
                return ActionPluginEdit.this;
            }

            public Object getMetaData() {
                return ejb.getMetaData();
            }

            public void updateMetaData(Object o, Plugin p) {
            }
        };
    }
}
